@@ -109,7 +109,7 @@ static int buf_size                                  = DEFAULT_PACKET_SIZE * 2;
 static int server_num                                = 0;
 static server_ctx_t *server_ctx_list[MAX_REMOTE_NUM] = { NULL };
 
-const char* s_port = NULL;
+const char *s_port = NULL;
 
 #ifndef __MINGW32__
 static int
@@ -252,8 +252,8 @@ parse_udprelay_header(const char *buf, const size_t buf_len,
             if (storage != NULL) {
                 struct sockaddr_in *addr = (struct sockaddr_in *)storage;
                 addr->sin_family = AF_INET;
-                addr->sin_addr   = *(struct in_addr *)(buf + offset);
-                addr->sin_port   = *(uint16_t *)(buf + offset + in_addr_len);
+                memcpy(&addr->sin_addr, buf + offset, sizeof(struct in_addr));
+                memcpy(&addr->sin_port, buf + offset + in_addr_len, sizeof(uint16_t));
             }
             if (host != NULL) {
                 inet_ntop(AF_INET, (const void *)(buf + offset),
@@ -273,12 +273,12 @@ parse_udprelay_header(const char *buf, const size_t buf_len,
                     if (ip.version == 4) {
                         struct sockaddr_in *addr = (struct sockaddr_in *)storage;
                         inet_pton(AF_INET, tmp, &(addr->sin_addr));
-                        addr->sin_port   = *(uint16_t *)(buf + offset + 1 + name_len);
+                        memcpy(&addr->sin_port, buf + offset + 1 + name_len, sizeof(uint16_t));
                         addr->sin_family = AF_INET;
                     } else if (ip.version == 6) {
                         struct sockaddr_in6 *addr = (struct sockaddr_in6 *)storage;
                         inet_pton(AF_INET, tmp, &(addr->sin6_addr));
-                        addr->sin6_port   = *(uint16_t *)(buf + offset + 1 + name_len);
+                        memcpy(&addr->sin6_port, buf + offset + 1 + name_len, sizeof(uint16_t));
                         addr->sin6_family = AF_INET6;
                     }
                 }
@@ -295,8 +295,8 @@ parse_udprelay_header(const char *buf, const size_t buf_len,
             if (storage != NULL) {
                 struct sockaddr_in6 *addr = (struct sockaddr_in6 *)storage;
                 addr->sin6_family = AF_INET6;
-                addr->sin6_addr   = *(struct in6_addr *)(buf + offset);
-                addr->sin6_port   = *(uint16_t *)(buf + offset + in6_addr_len);
+                memcpy(&addr->sin6_addr, buf + offset, sizeof(struct in6_addr));
+                memcpy(&addr->sin6_port, buf + offset + in6_addr_len, sizeof(uint16_t));
             }
             if (host != NULL) {
                 inet_ntop(AF_INET6, (const void *)(buf + offset),
@@ -312,7 +312,7 @@ parse_udprelay_header(const char *buf, const size_t buf_len,
     }
 
     if (port != NULL) {
-        sprintf(port, "%d", ntohs(*(uint16_t *)(buf + offset)));
+        sprintf(port, "%d", load16_be(buf + offset));
     }
     offset += 2;
 
@@ -327,19 +327,21 @@ get_addr_str(const struct sockaddr *sa)
     char addr[INET6_ADDRSTRLEN] = { 0 };
     char port[PORTSTRLEN]       = { 0 };
     uint16_t p;
+    struct sockaddr_in sa_in;
+    struct sockaddr_in6 sa_in6;
 
     switch (sa->sa_family) {
     case AF_INET:
-        inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr),
-                  addr, INET_ADDRSTRLEN);
-        p = ntohs(((struct sockaddr_in *)sa)->sin_port);
+        memcpy(&sa_in, sa, sizeof(struct sockaddr_in));
+        inet_ntop(AF_INET, &sa_in.sin_addr, addr, INET_ADDRSTRLEN);
+        p = ntohs(sa_in.sin_port);
         sprintf(port, "%d", p);
         break;
 
     case AF_INET6:
-        inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr),
-                  addr, INET6_ADDRSTRLEN);
-        p = ntohs(((struct sockaddr_in *)sa)->sin_port);
+        memcpy(&sa_in6, sa, sizeof(struct sockaddr_in6));
+        inet_ntop(AF_INET6, &sa_in6.sin6_addr, addr, INET6_ADDRSTRLEN);
+        p = ntohs(sa_in6.sin6_port);
         sprintf(port, "%d", p);
         break;
 
@@ -490,8 +492,8 @@ create_server_socket(const char *host, const char *port)
         }
 #ifdef IP_TOS
         // Set QoS flag
-        int tos = 46;
-        int proto = rp->ai_family == AF_INET6 ? IPPROTO_IP: IPPROTO_IPV6;
+        int tos   = 46;
+        int proto = rp->ai_family == AF_INET6 ? IPPROTO_IP : IPPROTO_IPV6;
         setsockopt(server_sock, proto, IP_TOS, &tos, sizeof(tos));
 #endif
 
@@ -652,8 +654,8 @@ resolv_cb(struct sockaddr *addr, void *data)
 #endif
 #ifdef IP_TOS
                 // Set QoS flag
-                int tos = 46;
-                int proto = addr->sa_family == AF_INET6 ? IPPROTO_IP: IPPROTO_IPV6;
+                int tos   = 46;
+                int proto = addr->sa_family == AF_INET6 ? IPPROTO_IP : IPPROTO_IPV6;
                 setsockopt(remotefd, proto, IP_TOS, &tos, sizeof(tos));
 #endif
 #ifdef SET_INTERFACE
@@ -662,9 +664,9 @@ resolv_cb(struct sockaddr *addr, void *data)
                         ERROR("setinterface");
                 }
 #endif
-                remote_ctx                  = new_remote(remotefd, query_ctx->server_ctx);
-                remote_ctx->src_addr        = query_ctx->src_addr;
-                remote_ctx->server_ctx      = query_ctx->server_ctx;
+                remote_ctx             = new_remote(remotefd, query_ctx->server_ctx);
+                remote_ctx->src_addr   = query_ctx->src_addr;
+                remote_ctx->server_ctx = query_ctx->server_ctx;
             } else {
                 ERROR("[udp] bind() error");
             }
@@ -840,8 +842,8 @@ remote_recv_cb(EV_P_ ev_io *w, int revents)
     }
 #ifdef IP_TOS
     // Set QoS flag
-    int tos = 46;
-    int proto = remote_ctx->src_addr.ss_family == AF_INET6 ? IPPROTO_IP: IPPROTO_IPV6;
+    int tos   = 46;
+    int proto = remote_ctx->src_addr.ss_family == AF_INET6 ? IPPROTO_IP : IPPROTO_IPV6;
     setsockopt(src_fd, proto, IP_TOS, &tos, sizeof(tos));
 #endif
     if (bind(src_fd, (struct sockaddr *)&dst_addr, remote_dst_addr_len) != 0) {
@@ -1012,7 +1014,7 @@ server_recv_cb(EV_P_ ev_io *w, int revents)
 
 #ifdef MODULE_REDIR
     char addr_header[MAX_ADDR_HEADER_SIZE] = { 0 };
-    int addr_header_len   = construct_udprelay_header(&dst_addr, addr_header);
+    int addr_header_len                    = construct_udprelay_header(&dst_addr, addr_header);
 
     if (addr_header_len == 0) {
         LOGE("[udp] failed to parse tproxy addr");
@@ -1028,11 +1030,11 @@ server_recv_cb(EV_P_ ev_io *w, int revents)
 #elif MODULE_TUNNEL
 
     char addr_header[MAX_ADDR_HEADER_SIZE] = { 0 };
-    char *host            = server_ctx->tunnel_addr.host;
-    char *port            = server_ctx->tunnel_addr.port;
-    uint16_t port_num     = (uint16_t)atoi(port);
-    uint16_t port_net_num = htons(port_num);
-    int addr_header_len   = 0;
+    char *host                             = server_ctx->tunnel_addr.host;
+    char *port                             = server_ctx->tunnel_addr.port;
+    uint16_t port_num                      = (uint16_t)atoi(port);
+    uint16_t port_net_num                  = htons(port_num);
+    int addr_header_len                    = 0;
 
     struct cork_ip ip;
     if (cork_ip_init(&ip, host) != -1) {
@@ -1084,7 +1086,7 @@ server_recv_cb(EV_P_ ev_io *w, int revents)
 #else
 
     char host[MAX_HOSTNAME_LEN] = { 0 };
-    char port[MAX_PORT_STR_LEN]  = { 0 };
+    char port[MAX_PORT_STR_LEN] = { 0 };
     struct sockaddr_storage dst_addr;
     memset(&dst_addr, 0, sizeof(struct sockaddr_storage));
 
@@ -1262,8 +1264,8 @@ server_recv_cb(EV_P_ ev_io *w, int revents)
 #endif
 #ifdef IP_TOS
                 // Set QoS flag
-                int tos = 46;
-                int proto = dst_addr.ss_family == AF_INET6 ? IPPROTO_IP: IPPROTO_IPV6;
+                int tos   = 46;
+                int proto = dst_addr.ss_family == AF_INET6 ? IPPROTO_IP : IPPROTO_IPV6;
                 setsockopt(remotefd, proto, IP_TOS, &tos, sizeof(tos));
 #endif
 #ifdef SET_INTERFACE
@@ -1272,9 +1274,9 @@ server_recv_cb(EV_P_ ev_io *w, int revents)
                         ERROR("setinterface");
                 }
 #endif
-                remote_ctx                  = new_remote(remotefd, server_ctx);
-                remote_ctx->src_addr        = src_addr;
-                remote_ctx->server_ctx      = server_ctx;
+                remote_ctx             = new_remote(remotefd, server_ctx);
+                remote_ctx->src_addr   = src_addr;
+                remote_ctx->server_ctx = server_ctx;
                 memcpy(&remote_ctx->dst_addr, &dst_addr, sizeof(struct sockaddr_storage));
             } else {
                 ERROR("[udp] bind() error");
